@@ -875,6 +875,9 @@ show_version()
     /* copyright string */
     if( (s=strusage(14)) )
 	printf("%s\n", s );
+    /* Licence string.  */
+    if( (s=strusage (10)) )
+        printf ("%s\n", s);
     /* copying conditions */
     if( (s=strusage(15)) )
 	fputs(s, stdout);
@@ -911,6 +914,7 @@ usage( int level )
  *     0: Copyright String auf stderr ausgeben
  *     1: Kurzusage auf stderr ausgeben und beenden
  *     2: Langusage auf stdout ausgeben und beenden
+ *    10: Return license info string
  *    11: name of program
  *    12: optional name of package which includes this program.
  *    13: version  string
@@ -934,26 +938,27 @@ strusage( int level )
 	return p;
 
     switch( level ) {
+      case 10: p = ("License GPLv3+: GNU GPL version 3 or later "
+                  "<http://gnu.org/licenses/gpl.html>");
+               break;
       case 11: p = "foo"; break;
       case 13: p = "0.0"; break;
       case 14: p = "Copyright (C) 2010 Free Software Foundation, Inc."; break;
       case 15: p =
-"This program comes with ABSOLUTELY NO WARRANTY.\n"
-"This is free software, and you are welcome to redistribute it\n"
-"under certain conditions. See the file COPYING for details.\n"; break;
+"This is free software: you are free to change and redistribute it.\n"
+"There is NO WARRANTY, to the extent permitted by law.\n";
+        break;
       case 16:	p =
 "This is free software; you can redistribute it and/or modify\n"
 "it under the terms of the GNU General Public License as published by\n"
-"the Free Software Foundation; either version 2 of the License, or\n"
+"the Free Software Foundation; either version 3 of the License, or\n"
 "(at your option) any later version.\n\n"
 "It is distributed in the hope that it will be useful,\n"
 "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
 "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
 "GNU General Public License for more details.\n\n"
 "You should have received a copy of the GNU General Public License\n"
-"along with this program; if not, write to the Free Software\n"
-"Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,\n"
-"USA.\n";
+"along with this software.  If not, see <http://www.gnu.org/licenses/>.\n";
 	break;
       case 40: /* short and long usage */
       case 41: p = ""; break;
