@@ -1,12 +1,12 @@
 /* dirmngr-client.c  -  A client for the dirmngr daemon
- *	Copyright (C) 2004, 2007 g10 Code GmbH
+ *	Copyright (C) 2004, 2007, 2010 g10 Code GmbH
  *	Copyright (C) 2002, 2003 Free Software Foundation, Inc.
  *
  * This file is part of DirMngr.
  *
  * DirMngr is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * DirMngr is distributed in the hope that it will be useful,
@@ -80,7 +80,7 @@ static ARGPARSE_OPTS opts[] = {
   { oPEM,      "pem",       0, N_("certificates are expected in PEM format")},
   { oForceDefaultResponder, "force-default-responder", 0,
     N_("force the use of the default OCSP responder")},
-  { 0 }
+  { 0, NULL, 0, NULL }
 };
  
 
@@ -211,9 +211,8 @@ main (int argc, char **argv )
 #endif /*HAVE_W32_SYSTEM*/
   
   /* Init Assuan.  */
-  assuan_set_assuan_log_stream (log_get_stream ());
   assuan_set_assuan_log_prefix (log_get_prefix (NULL));
-  assuan_set_assuan_err_source (GPG_ERR_SOURCE_DEFAULT);
+  assuan_set_gpg_err_source (GPG_ERR_SOURCE_DEFAULT);
 
   /* Setup I18N. */
   i18n_init();
@@ -367,7 +366,7 @@ main (int argc, char **argv )
       xfree (certbuf);
     }
 
-  assuan_disconnect (ctx);
+  assuan_release (ctx);
 
   if (cmd_ping)
     {
@@ -419,7 +418,7 @@ main (int argc, char **argv )
 
 
 /* Print status line from the assuan protocol.  */
-static int
+static gpg_error_t
 status_cb (void *opaque, const char *line)
 {
   (void)opaque;
@@ -430,7 +429,7 @@ status_cb (void *opaque, const char *line)
 }
 
 /* Print data as retrieved by the lookup function.  */
-static int
+static gpg_error_t
 data_cb (void *opaque, const void *buffer, size_t length)
 {
   gpg_error_t err;
@@ -463,6 +462,15 @@ start_dirmngr (int only_daemon)
       infostr = xstrdup (default_socket_name ());
       try_default = 1;
     }
+
+  rc = assuan_new (&ctx);
+  if (rc)
+    {
+      log_error (_("failed to allocate assuan context: %s\n"),
+                 gpg_strerror (rc));
+      return NULL;
+    }
+
   if (!infostr || !*infostr)
     {
       const char *pgmname;
@@ -492,13 +500,13 @@ start_dirmngr (int only_daemon)
 
       i=0;
       if (log_get_fd () != -1)
-        no_close_list[i++] = log_get_fd ();
-      no_close_list[i++] = fileno (stderr);
+        no_close_list[i++] = assuan_fd_from_posix_fd (log_get_fd ());
+      no_close_list[i++] = assuan_fd_from_posix_fd (fileno (stderr));
       no_close_list[i] = -1;
 
       /* Connect to the agent and perform initial handshaking.  */
-      rc = assuan_pipe_connect (&ctx, opt.dirmngr_program, argv,
-                                no_close_list);
+      rc = assuan_pipe_connect (ctx, opt.dirmngr_program, argv,
+                                no_close_list, NULL, NULL, 0);
     }
   else /* Connect to a daemon.  */
     {
@@ -537,7 +545,7 @@ start_dirmngr (int only_daemon)
       else
         pid = -1;
 
-      rc = assuan_socket_connect (&ctx, infostr, pid);
+      rc = assuan_socket_connect (ctx, infostr, pid, 0);
       xfree (infostr);
       if (gpg_err_code(rc) == GPG_ERR_ASS_CONNECT_FAILED && !only_daemon)
         {
@@ -549,6 +557,7 @@ start_dirmngr (int only_daemon)
 
   if (rc)
     {
+      assuan_release (ctx);
       log_error (_("can't connect to the dirmngr: %s\n"),
                  gpg_strerror (rc));
       return NULL;
@@ -766,7 +775,7 @@ read_certificate (const char *fname, unsigned char **rbuf, size_t *rbuflen)
 
 
 /* Callback for the inquire fiunction to send back the certificate.  */
-static int
+static gpg_error_t
 inq_cert (void *opaque, const char *line)
 {
   struct inq_cert_parm_s *parm = opaque;
@@ -883,25 +892,32 @@ do_loadcrl (assuan_context_t ctx, const char *filename)
   const char *s;
   char *fname, *line, *p;
 
+  if (opt.url)
+    fname = xstrdup (filename);
+  else
+    {
 #ifdef HAVE_CANONICALIZE_FILE_NAME
-  fname = canonicalize_file_name (filename);
-  if (!fname)
-    {
-      log_error ("error canonicalizing `%s': %s\n",
-                 filename, strerror (errno));
-      return gpg_error (GPG_ERR_GENERAL);
-    }      
+      fname = canonicalize_file_name (filename);
+      if (!fname)
+        {
+          log_error ("error canonicalizing `%s': %s\n",
+                     filename, strerror (errno));
+          return gpg_error (GPG_ERR_GENERAL);
+        }      
 #else
-  fname = xstrdup (filename);
+      fname = xstrdup (filename);
 #endif
-  if (*fname != '/')
-    {
-      log_error (_("absolute file name expected\n"));
-      return gpg_error (GPG_ERR_GENERAL);
+      if (*fname != '/')
+        {
+          log_error (_("absolute file name expected\n"));
+          return gpg_error (GPG_ERR_GENERAL);
+        }
     }
-
-  line = xmalloc (8+ strlen (fname) * 3 + 1);
+  
+  line = xmalloc (8 + 6 + strlen (fname) * 3 + 1);
   p = stpcpy (line, "LOADCRL ");
+  if (opt.url)
+    p = stpcpy (p, "--url ");
   for (s = fname; *s; s++)
     {
       if (*s < ' ' || *s == '+')
@@ -913,7 +929,7 @@ do_loadcrl (assuan_context_t ctx, const char *filename)
         *p++ = '+';
       else
         *p++ = *s;
-    }
+        }
   *p = 0;
 
   err = assuan_transact (ctx, line, NULL, NULL,
@@ -1021,5 +1037,7 @@ squid_loop_body (assuan_context_t ctx)
       puts ("ERROR");
     }
   
+  fflush (stdout);
+
   return 0;
 }
