@@ -1,12 +1,12 @@
 /* dirmngr.c - LDAP access
  *	Copyright (C) 2002 Klarälvdalens Datakonsult AB
- *      Copyright (C) 2003, 2004, 2006, 2007, 2008 g10 Code GmbH
+ *      Copyright (C) 2003, 2004, 2006, 2007, 2008, 2010 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
  * DirMngr is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * DirMngr is distributed in the hope that it will be useful,
@@ -60,14 +60,23 @@
 #define FD2INT(h) (h)
 #endif
 
-enum cmd_and_opt_values 
+#ifndef HAVE_SIGSET_T
+# ifdef _WIN64
+    typedef unsigned long long sigset_t;
+# else
+    typedef unsigned long sigset_t;
+# endif
+#endif
+
+
+enum cmd_and_opt_values
 { aNull = 0,
   oCsh		  = 'c',
   oQuiet	  = 'q',
   oSh		  = 's',
   oVerbose	  = 'v',
   oNoVerbose = 500,
-  
+
   aServer,
   aDaemon,
   aService,
@@ -114,12 +123,13 @@ enum cmd_and_opt_values
   oSocketName,
   oLDAPWrapperProgram,
   oHTTPWrapperProgram,
+  oIgnoreCertExtension,
 aTest };
 
 
 
 static ARGPARSE_OPTS opts[] = {
-  
+
   { 300, NULL, 0, N_("@Commands:\n ") },
 
   { aServer,   "server",    256, N_("run in server mode (foreground)") },
@@ -165,7 +175,7 @@ static ARGPARSE_OPTS opts[] = {
     N_("|URL|redirect all HTTP requests to URL")},
   { oLDAPProxy,  "ldap-proxy", 2,
     N_("|HOST|use HOST for LDAP queries")},
-  { oOnlyLDAPProxy, "only-ldap-proxy", 0, 
+  { oOnlyLDAPProxy, "only-ldap-proxy", 0,
     N_("do not use fallback hosts with --ldap-proxy")},
 
   { oLDAPFile, "ldapserverlist-file", 2,
@@ -176,7 +186,7 @@ static ARGPARSE_OPTS opts[] = {
     N_("|N|set LDAP timeout to N seconds")},
 
   { oOCSPResponder, "ocsp-responder", 2, N_("|URL|use OCSP responder at URL")},
-  { oOCSPSigner, "ocsp-signer", 2, N_("|FPR|OCSP response signed by FPR")}, 
+  { oOCSPSigner, "ocsp-signer", 2, N_("|FPR|OCSP response signed by FPR")},
   { oOCSPMaxClockSkew, "ocsp-max-clock-skew", 1, "@" },
   { oOCSPMaxPeriod, "ocsp-max-period", 1, "@" },
   { oOCSPCurrentPeriod, "ocsp-current-period", 1, "@" },
@@ -191,14 +201,14 @@ static ARGPARSE_OPTS opts[] = {
   { oDebugAll, "debug-all" ,0,    "@"},
   { oDebugWait, "debug-wait", 1, "@"},
   { oNoGreeting, "no-greeting", 0, "@"},
-  { oHomedir, "homedir", 2, "@" },  
+  { oHomedir, "homedir", 2, "@" },
   { oLDAPWrapperProgram, "ldap-wrapper-program", 2, "@"},
   { oHTTPWrapperProgram, "http-wrapper-program", 2, "@"},
   { oHonorHTTPProxy,     "honor-http-proxy", 0, "@" },
+  { oIgnoreCertExtension,"ignore-cert-extension", 2, "@"},
 
-  { 302, NULL, 0, N_(
-  "@\n(See the \"info\" manual for a complete listing of all commands and options)\n"
-                    )},
+  { 302, NULL, 0, N_("@\n(See the \"info\" manual for a complete listing "
+                     "of all commands and options)\n")},
 
   { 0, NULL, 0, NULL }
 };
@@ -238,6 +248,24 @@ static int active_connections;
 #define TIMERTICK_INTERVAL    (2)    /* Seconds.  */
 #endif
 
+/* This union is used to avoid compiler warnings in case a pointer is
+   64 bit and an int 32 bit.  We store an integer in a pointer and get
+   it back later (pth_key_getdata et al.).  */
+union int_and_ptr_u
+{
+  int  aint;
+  assuan_fd_t afd;
+  void *aptr;
+};
+
+
+
+/* The key used to store the current file descriptor in the thread
+   local storage.  We use this in conjunction with the
+   log_set_pid_suffix_cb feature..  */
+#ifndef HAVE_W32_SYSTEM
+static int my_tlskey_current_fd;
+#endif
 
 /* Prototypes. */
 static void cleanup (void);
@@ -246,14 +274,15 @@ static fingerprint_list_t parse_ocsp_signer (const char *string);
 static void handle_connections (assuan_fd_t listen_fd);
 
 /* Pth wrapper function definitions. */
-GCRY_THREAD_OPTION_PTH_IMPL;
+ASSUAN_SYSTEM_PTH_IMPL;
 
+GCRY_THREAD_OPTION_PTH_IMPL;
 
 static const char *
 my_strusage( int level )
 {
   const char *p;
-  switch ( level ) 
+  switch ( level )
     {
     case 11: p = "dirmngr";
       break;
@@ -271,7 +300,7 @@ my_strusage( int level )
     case 41: p = _("Syntax: dirmngr [options] [command [args]]\n"
                    "LDAP and OCSP access for GnuPG\n");
       break;
-      
+
     default: p = NULL;
     }
   return p;
@@ -294,7 +323,7 @@ my_gcry_logger (void *dummy, int level, const char *fmt, va_list arg_ptr)
     case GCRY_LOG_FATAL:level = JNLIB_LOG_FATAL; break;
     case GCRY_LOG_BUG:  level = JNLIB_LOG_BUG; break;
     case GCRY_LOG_DEBUG:level = JNLIB_LOG_DEBUG; break;
-    default:            level = JNLIB_LOG_ERROR; break;  
+    default:            level = JNLIB_LOG_ERROR; break;
     }
   log_logv (level, fmt, arg_ptr);
 }
@@ -304,18 +333,18 @@ my_gcry_logger (void *dummy, int level, const char *fmt, va_list arg_ptr)
    implementation does only allow SHA-1 for hashing. This may be
    extended by mapping the name, testing for algorithm availibility
    and adjust the length checks accordingly. */
-static gpg_error_t 
+static gpg_error_t
 my_ksba_hash_buffer (void *arg, const char *oid,
                      const void *buffer, size_t length, size_t resultsize,
                      unsigned char *result, size_t *resultlen)
 {
   (void)arg;
 
-  if (oid && strcmp (oid, "1.3.14.3.2.26")) 
-    return gpg_error (GPG_ERR_NOT_SUPPORTED); 
+  if (oid && strcmp (oid, "1.3.14.3.2.26"))
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
   if (resultsize < 20)
     return gpg_error (GPG_ERR_BUFFER_TOO_SHORT);
-  gcry_md_hash_buffer (2, result, buffer, length); 
+  gcry_md_hash_buffer (2, result, buffer, length);
   *resultlen = 20;
   return 0;
 }
@@ -328,19 +357,30 @@ my_ksba_hash_buffer (void *arg, const char *oid,
 static void
 set_debug (void)
 {
+  int numok = (debug_level && digitp (debug_level));
+  int numlvl = numok? atoi (debug_level) : 0;
+
   if (!debug_level)
     ;
-  else if (!strcmp (debug_level, "none"))
+  else if (!strcmp (debug_level, "none") || (numok && numlvl < 1))
     opt.debug = 0;
-  else if (!strcmp (debug_level, "basic"))
+  else if (!strcmp (debug_level, "basic") || (numok && numlvl <= 2))
     opt.debug = DBG_ASSUAN_VALUE;
-  else if (!strcmp (debug_level, "advanced"))
+  else if (!strcmp (debug_level, "advanced") || (numok && numlvl <= 5))
     opt.debug = (DBG_ASSUAN_VALUE|DBG_X509_VALUE|DBG_LOOKUP_VALUE);
-  else if (!strcmp (debug_level, "expert"))
+  else if (!strcmp (debug_level, "expert") || (numok && numlvl <= 8))
     opt.debug = (DBG_ASSUAN_VALUE|DBG_X509_VALUE|DBG_LOOKUP_VALUE
                  |DBG_CACHE_VALUE|DBG_CRYPTO_VALUE);
-  else if (!strcmp (debug_level, "guru"))
-    opt.debug = ~0;
+  else if (!strcmp (debug_level, "guru") || numok)
+    {
+      opt.debug = ~0;
+      /* Unless the "guru" string has been used we don't want to allow
+         hashing debugging.  The rationale is that people tend to
+         select the highest debug value and would then clutter their
+         disk with debug files which may reveal confidential data.  */
+      if (numok)
+        opt.debug &= ~(DBG_HASHING_VALUE);
+    }
   else
     {
       log_error (_("invalid debug-level `%s' given\n"), debug_level);
@@ -362,7 +402,7 @@ set_debug (void)
   if (opt.debug & DBG_CRYPTO_VALUE )
     gcry_control (GCRYCTL_SET_DEBUG_FLAGS, 1);
 }
- 
+
 
 static void
 i18n_init(void)
@@ -436,10 +476,10 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
       opt.debug = 0;
       opt.ldap_wrapper_program = NULL;
       opt.disable_http = 0;
-      opt.disable_ldap = 0; 
-      opt.honor_http_proxy = 0; 
-      opt.http_proxy = NULL; 
-      opt.ldap_proxy = NULL; 
+      opt.disable_ldap = 0;
+      opt.honor_http_proxy = 0;
+      opt.http_proxy = NULL;
+      opt.ldap_proxy = NULL;
       opt.only_ldap_proxy = 0;
       opt.ignore_http_dp = 0;
       opt.ignore_ldap_dp = 0;
@@ -456,6 +496,7 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
           xfree (opt.ocsp_signer);
           opt.ocsp_signer = tmp;
         }
+      FREE_STRLIST (opt.ignored_cert_extensions);
       return 1;
     }
 
@@ -498,7 +539,7 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
 
     case oAllowOCSP: opt.allow_ocsp = 1; break;
     case oOCSPResponder: opt.ocsp_responder = pargs->r.ret_str; break;
-    case oOCSPSigner: 
+    case oOCSPSigner:
       opt.ocsp_signer = parse_ocsp_signer (pargs->r.ret_str);
       break;
     case oOCSPMaxClockSkew: opt.ocsp_max_clock_skew = pargs->r.ret_int; break;
@@ -507,6 +548,10 @@ parse_rereadable_options (ARGPARSE_ARGS *pargs, int reread)
 
     case oMaxReplies: opt.max_replies = pargs->r.ret_int; break;
 
+    case oIgnoreCertExtension:
+      add_to_strlist (&opt.ignored_cert_extensions, pargs->r.ret_str);
+      break;
+
     default:
       return 0; /* Not handled. */
     }
@@ -530,13 +575,13 @@ w32_service_control (DWORD control, DWORD event_type, LPVOID event_data,
     case SERVICE_CONTROL_SHUTDOWN:
       /* For shutdown we will try to force termination.  */
       service_status.dwCurrentState = SERVICE_STOP_PENDING;
-      SetServiceStatus (service_handle, &service_status); 
+      SetServiceStatus (service_handle, &service_status);
       shutdown_pending = 3;
       break;
 
     case SERVICE_CONTROL_STOP:
       service_status.dwCurrentState = SERVICE_STOP_PENDING;
-      SetServiceStatus (service_handle, &service_status); 
+      SetServiceStatus (service_handle, &service_status);
       shutdown_pending = 1;
       break;
 
@@ -547,6 +592,18 @@ w32_service_control (DWORD control, DWORD event_type, LPVOID event_data,
 }
 #endif /*HAVE_W32_SYSTEM*/
 
+#ifndef HAVE_W32_SYSTEM
+static int
+pid_suffix_callback (int *r_suffix)
+{
+  union int_and_ptr_u value;
+
+  value.aptr = pth_key_getdata (my_tlskey_current_fd);
+  *r_suffix = value.aint;
+  return (*r_suffix != -1);
+}
+#endif /*!HAVE_W32_SYSTEM*/
+
 
 #ifdef HAVE_W32_SYSTEM
 #define main real_main
@@ -576,6 +633,7 @@ main (int argc, char **argv)
   int debug_wait = 0;
   int rc;
   int homedir_seen = 0;
+  struct assuan_malloc_hooks malloc_hooks;
 
 #ifdef HAVE_W32_SYSTEM
   /* The option will be set by main() below if we should run as a
@@ -596,13 +654,13 @@ main (int argc, char **argv)
       service_status.dwServiceSpecificExitCode = NO_ERROR;
       service_status.dwCheckPoint = 0;
       service_status.dwWaitHint = 10000; /* 10 seconds timeout.  */
-      SetServiceStatus (service_handle, &service_status); 
+      SetServiceStatus (service_handle, &service_status);
     }
 #endif /*HAVE_W32_SYSTEM*/
 
   set_strusage (my_strusage);
-  log_set_prefix ("dirmngr", 1|4); 
-  
+  log_set_prefix ("dirmngr", 1|4);
+
   /* Check that the libraries are suitable.  Do it here because
      the option parsing may need services of the libraries. */
 
@@ -638,10 +696,14 @@ main (int argc, char **argv)
 
 
   /* Init Assuan. */
-  assuan_set_malloc_hooks (gcry_malloc, gcry_realloc, gcry_free);
-  assuan_set_assuan_log_stream (log_get_stream ());
+  malloc_hooks.malloc = gcry_malloc;
+  malloc_hooks.realloc = gcry_realloc;
+  malloc_hooks.free = gcry_free;
+  assuan_set_malloc_hooks (&malloc_hooks);
   assuan_set_assuan_log_prefix (log_get_prefix (NULL));
-  assuan_set_assuan_err_source (GPG_ERR_SOURCE_DEFAULT);
+  assuan_set_gpg_err_source (GPG_ERR_SOURCE_DEFAULT);
+  assuan_set_system_hooks (ASSUAN_SYSTEM_PTH);
+  assuan_sock_init ();
 
   /* Setup I18N. */
   i18n_init();
@@ -650,11 +712,20 @@ main (int argc, char **argv)
   shell = getenv ("SHELL");
   if (shell && strlen (shell) >= 3 && !strcmp (shell+strlen (shell)-3, "csh") )
     csh_style = 1;
-  
+
   opt.homedir = default_homedir ();
 
+  /* Now with Pth running we can set the logging callback.  Our
+     windows implementation does not yet feature the PTh tls
+     functions.  */
+#ifndef HAVE_W32_SYSTEM
+  if (pth_key_create (&my_tlskey_current_fd, NULL))
+    if (pth_key_setdata (my_tlskey_current_fd, NULL))
+      log_set_pid_suffix_cb (pid_suffix_callback);
+#endif /*!HAVE_W32_SYSTEM*/
+
   /* Reset rereadable options to default values. */
-  parse_rereadable_options (NULL, 0); 
+  parse_rereadable_options (NULL, 0);
 
   /* LDAP defaults.  */
   opt.add_new_ldapservers = 0;
@@ -716,7 +787,7 @@ main (int argc, char **argv)
 
   if (default_config)
     configname = make_filename (opt.homedir, "dirmngr.conf", NULL );
-  
+
   argc = orig_argc;
   argv = orig_argv;
   pargs.argc = &argc;
@@ -741,7 +812,7 @@ main (int argc, char **argv)
                          configname, strerror(errno) );
               exit(2);
 	    }
-          xfree (configname); 
+          xfree (configname);
           configname = NULL;
 	}
       if (parse_debug && configname )
@@ -755,13 +826,13 @@ main (int argc, char **argv)
         continue; /* Already handled */
       switch (pargs.r_opt)
         {
-        case aServer: 
+        case aServer:
         case aDaemon:
         case aService:
-        case aShutdown: 
-        case aFlush: 
-	case aListCRLs: 
-	case aLoadCRL: 
+        case aShutdown:
+        case aFlush:
+	case aListCRLs:
+	case aLoadCRL:
         case aFetchCRL:
 	case aGPGConfList:
 	case aGPGConfTest:
@@ -796,8 +867,8 @@ main (int argc, char **argv)
         case oSh: csh_style = 0; break;
 	case oLDAPFile: ldapfile = pargs.r.ret_str; break;
 	case oLDAPAddServers: opt.add_new_ldapservers = 1; break;
-	case oLDAPTimeout: 
-	  opt.ldaptimeout = pargs.r.ret_int; 
+	case oLDAPTimeout:
+	  opt.ldaptimeout = pargs.r.ret_int;
 	  break;
 
         case oFakedSystemTime:
@@ -853,7 +924,7 @@ main (int argc, char **argv)
   set_debug ();
 
   /* Get LDAP server list from file. */
-  if (!ldapfile) 
+  if (!ldapfile)
     {
       ldapfile = make_filename (opt.homedir,
                                 opt.system_daemon?
@@ -889,9 +960,8 @@ main (int argc, char **argv)
         {
           log_set_file (logfile);
           log_set_prefix (NULL, 2|4);
-	  assuan_set_assuan_log_stream (log_get_stream ());
         }
-       
+
       if (debug_wait)
         {
           log_debug ("waiting for debugger - my pid is %u .....\n",
@@ -915,7 +985,7 @@ main (int argc, char **argv)
 
       if (argc)
         wrong_args ("--daemon");
-      
+
       /* Now start with logging to a file if this is desired. */
       if (logfile)
         {
@@ -924,7 +994,6 @@ main (int argc, char **argv)
                                  |JNLIB_LOG_WITH_TIME
                                  |JNLIB_LOG_WITH_PID));
           current_logfile = xstrdup (logfile);
-	  assuan_set_assuan_log_stream (log_get_stream ());
         }
 
 #ifndef HAVE_W32_SYSTEM
@@ -934,12 +1003,12 @@ main (int argc, char **argv)
           dirmngr_exit (1);
         }
 #endif
-      if (strlen (socket_name)+1 >= sizeof serv_addr.sun_path ) 
+      if (strlen (socket_name)+1 >= sizeof serv_addr.sun_path )
         {
           log_error (_("name of socket too long\n"));
           dirmngr_exit (1);
         }
-    
+
       fd = assuan_sock_new (AF_UNIX, SOCK_STREAM, 0);
       if (fd == ASSUAN_INVALID_FD)
         {
@@ -960,7 +1029,7 @@ main (int argc, char **argv)
 	  remove (socket_name);
 	  rc = assuan_sock_bind (fd, (struct sockaddr*) &serv_addr, len);
 	}
-      if (rc != -1 
+      if (rc != -1
 	  && (rc = assuan_sock_get_nonce ((struct sockaddr*) &serv_addr, len, &socket_nonce)))
 	log_error (_("error getting nonce for the socket\n"));
       if (rc == -1)
@@ -971,7 +1040,7 @@ main (int argc, char **argv)
           dirmngr_exit (1);
         }
       cleanup_socket = 1;
-  
+
       if (listen (FD2INT (fd), 5) == -1)
         {
           log_error (_("listen() failed: %s\n"), strerror (errno));
@@ -989,22 +1058,22 @@ main (int argc, char **argv)
       printf ("set DIRMNGR_INFO=%s;%lu;1\n", socket_name, (ulong) pid);
 #else
       pid = pth_fork ();
-      if (pid == (pid_t)-1) 
+      if (pid == (pid_t)-1)
         {
           log_fatal (_("fork failed: %s\n"), strerror (errno) );
           dirmngr_exit (1);
         }
 
-      if (pid) 
+      if (pid)
         { /* We are the parent */
           char *infostr;
-          
+
           /* Don't let cleanup() remove the socket - the child is
              responsible for doing that.  */
           cleanup_socket = 0;
 
           close (fd);
-          
+
           /* Create the info string: <name>:<pid>:<protocol_version> */
           if (asprintf (&infostr, "DIRMNGR_INFO=%s:%lu:1",
                         socket_name, (ulong)pid ) < 0)
@@ -1025,18 +1094,18 @@ main (int argc, char **argv)
               printf ( "%s; export DIRMNGR_INFO;\n", infostr);
             }
           free (infostr);
-          exit (0); 
+          exit (0);
           /*NEVER REACHED*/
         } /* end parent */
-      
-      
-      /* 
+
+
+      /*
          This is the child
        */
 
       /* Detach from tty and put process into a new session */
       if (!nodetach )
-        { 
+        {
           int i;
           unsigned int oldflags;
 
@@ -1132,7 +1201,7 @@ main (int argc, char **argv)
                      argv[0], gpg_strerror (rc));
       else
         {
-          rc = crl_cache_insert (&ctrlbuf, argv[0], reader); 
+          rc = crl_cache_insert (&ctrlbuf, argv[0], reader);
           if (rc)
             log_error (_("processing CRL from `%s' failed: %s\n"),
                        argv[0], gpg_strerror (rc));
@@ -1203,7 +1272,7 @@ main (int argc, char **argv)
          and having both of them is thus problematic.  --no-detach is
          also only usable on the command line.  --batch is unused.  */
 
-      filename = make_filename (opt.homedir, 
+      filename = make_filename (opt.homedir,
                                 opt.system_daemon?
                                 "ldapservers.conf":"dirmngr_ldapservers.conf",
                                 NULL);
@@ -1294,7 +1363,7 @@ cleanup (void)
 }
 
 
-void 
+void
 dirmngr_exit (int rc)
 {
   cleanup ();
@@ -1312,7 +1381,7 @@ dirmngr_init_default_ctrl (ctrl_t ctrl)
 
 
 /* Create a list of LDAP servers from the file FILENAME. Returns the
-   list or NULL in case of errors. 
+   list or NULL in case of errors.
 
    The format fo such a file is line oriented where empty lines and
    lines starting with a hash mark are ignored.  All other lines are
@@ -1320,7 +1389,7 @@ dirmngr_init_default_ctrl (ctrl_t ctrl)
 
    1. field: Hostname
    2. field: Portnumber
-   3. field: Username 
+   3. field: Username
    4. field: Password
    5. field: Base DN
 
@@ -1373,8 +1442,8 @@ parse_ldapserver_file (const char* filename)
           *serverend = server;
           serverend = &server->next;
         }
-    } 
-  
+    }
+
   if (ferror (fp))
     log_error (_("error reading `%s': %s\n"), filename, strerror (errno));
   fclose (fp);
@@ -1408,14 +1477,14 @@ parse_ocsp_signer (const char *string)
       item->hexfpr[j] = 0;
       if (j != 40 || !(spacep (string+i) || !string[i]))
         {
-          log_error (_("%s:%u: invalid fingerprint detected\n"), 
+          log_error (_("%s:%u: invalid fingerprint detected\n"),
                      "--ocsp-signer", 0);
           xfree (item);
           return NULL;
         }
       return item;
     }
-  
+
   /* Well, it is a filename.  */
   if (*string == '/' || (*string == '~' && string[1] == '/'))
     fname = make_filename (string, NULL);
@@ -1470,7 +1539,7 @@ parse_ocsp_signer (const char *string)
             ;
           err = gpg_error (*line? GPG_ERR_LINE_TOO_LONG
                            /* */: GPG_ERR_INCOMPLETE_LINE);
-          log_error (_("%s:%u: read error: %s\n"), 
+          log_error (_("%s:%u: read error: %s\n"),
                      fname, lnr, gpg_strerror (err));
           errflag = 1;
           continue;
@@ -1508,14 +1577,14 @@ parse_ocsp_signer (const char *string)
 
 
 /*
-   Stuff used in daemon mode.  
+   Stuff used in daemon mode.
  */
 
 
 
 /* Reread parts of the configuration.  Note, that this function is
    obviously not thread-safe and should only be called from the PTH
-   signal handler. 
+   signal handler.
 
    Fixme: Due to the way the argument parsing works, we create a
    memory leak here for all string type arguments.  There is currently
@@ -1577,11 +1646,11 @@ handle_signal (int signo)
       cert_cache_init ();
       crl_cache_init ();
       break;
-      
+
     case SIGUSR1:
       cert_cache_print_stats ();
       break;
-      
+
     case SIGUSR2:
       log_info (_("SIGUSR2 received - no action defined\n"));
       break;
@@ -1601,7 +1670,7 @@ handle_signal (int signo)
           dirmngr_exit (0);
 	}
       break;
-        
+
     case SIGINT:
       log_info (_("SIGINT received - immediate shutdown\n"));
       log_info( "%s %s stopped\n", strusage(11), strusage(13));
@@ -1639,12 +1708,12 @@ handle_tick (void)
 
 /* Check the nonce on a new connection.  This is a NOP unless we we
    are using our Unix domain socket emulation under Windows.  */
-static int 
+static int
 check_nonce (assuan_fd_t fd, assuan_sock_nonce_t *nonce)
 {
   if (assuan_sock_check_nonce (fd, nonce))
     {
-      log_info (_("error reading nonce on fd %d: %s\n"), 
+      log_info (_("error reading nonce on fd %d: %s\n"),
                 FD2INT (fd), strerror (errno));
       assuan_sock_close (fd);
       return -1;
@@ -1658,11 +1727,19 @@ check_nonce (assuan_fd_t fd, assuan_sock_nonce_t *nonce)
 static void *
 start_connection_thread (void *arg)
 {
-  assuan_fd_t fd = (assuan_fd_t) arg;
+  union int_and_ptr_u argval;
+  assuan_fd_t fd;
+
+  argval.aptr = arg;
+  fd = argval.afd;
 
   if (check_nonce (fd, &socket_nonce))
     return NULL;
 
+#ifndef HAVE_W32_SYSTEM
+  pth_key_setdata (my_tlskey_current_fd, argval.aptr);
+#endif
+
   active_connections++;
   if (opt.verbose)
     log_info (_("handler for fd %d started\n"), FD2INT (fd));
@@ -1672,7 +1749,12 @@ start_connection_thread (void *arg)
   if (opt.verbose)
     log_info (_("handler for fd %d terminated\n"), FD2INT (fd));
   active_connections--;
-  
+
+#ifndef HAVE_W32_SYSTEM
+  argval.afd = ASSUAN_INVALID_FD;
+  pth_key_setdata (my_tlskey_current_fd, argval.aptr);
+#endif
+
   return NULL;
 }
 
@@ -1725,7 +1807,7 @@ handle_connections (assuan_fd_t listen_fd)
 	}
 
       if (!time_ev)
-        time_ev = pth_event (PTH_EVENT_TIME, 
+        time_ev = pth_event (PTH_EVENT_TIME,
                              pth_timeout (TIMERTICK_INTERVAL, 0));
 
       if (time_ev)
@@ -1773,17 +1855,22 @@ handle_connections (assuan_fd_t listen_fd)
       pth_sigmask (SIG_BLOCK, &sigs, &oldsigs);
 
       /* Create thread to handle this connection.  */
-      if (!pth_spawn (tattr, start_connection_thread, (void*)fd))
-        {
-          log_error (_("error spawning connection handler: %s\n"),
-                     strerror (errno) );
-          assuan_sock_close (fd);
-	}
+      {
+        union int_and_ptr_u argval;
+
+        argval.afd = fd;
+        if (!pth_spawn (tattr, start_connection_thread, argval.aptr))
+          {
+            log_error (_("error spawning connection handler: %s\n"),
+                       strerror (errno) );
+            assuan_sock_close (fd);
+          }
+      }
 
       /* Restore the signal mask. */
       pth_sigmask (SIG_SETMASK, &oldsigs, NULL);
     }
-  
+
   pth_event_free (ev, PTH_FREE_ALL);
   if (time_ev)
     pth_event_free (time_ev, PTH_FREE_ALL);
