@@ -236,6 +236,39 @@ w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
     return -1;
 }
 
+
+#if 0
+static char *
+find_program_in_inst_dir (const char *name)
+{
+  char *result = NULL;
+  char *tmp;
+
+  tmp = read_w32_registry_string ("HKEY_LOCAL_MACHINE",
+				  "Software\\GNU\\GnuPG",
+				  "Install Directory");
+  if (!tmp)
+    return NULL;
+
+  result = malloc (strlen (tmp) + 1 + strlen (name) + 1);
+  if (!result)
+    {
+      free (tmp);
+      return NULL;
+    }
+
+  strcpy (stpcpy (stpcpy (result, tmp), "\\"), name);
+  free (tmp);
+  if (access (result, F_OK))
+    {
+      free (result);
+      return NULL;
+    }
+
+  return result;
+}
+
+
 static char *
 find_program_at_standard_place (const char *name)
 {
@@ -258,16 +291,23 @@ find_program_at_standard_place (const char *name)
   return result;
 }
 #endif
+#endif
 
 
 const char *
 get_dirmngr_ldap_path (void)
 {
-  static const char *pgmname;
+  static char *pgmname;
 
 #ifdef HAVE_W32_SYSTEM
-  if (!pgmname)
-    pgmname = find_program_at_standard_place ("GNU\\GnuPG\\dirmngr_ldap.exe");
+  if (! pgmname)
+    {
+      const char *dir = dirmngr_libexecdir ();
+      const char *exe = "\\dirmngr_ldap.exe";
+      pgmname = malloc (strlen (dir) + strlen (exe) + 1);
+      if (pgmname)
+	strcpy (stpcpy (pgmname, dir), exe);
+    }
 #endif
   if (!pgmname)
     pgmname = DIRMNGR_LIBEXECDIR "/dirmngr_ldap";
@@ -512,7 +552,7 @@ dirmngr_cachedir (void)
       char path[MAX_PATH];
       const char *s1[] = { "GNU", "cache", "dirmngr", NULL };
       int s1_len;
-      char **comp;
+      const char **comp;
 
       s1_len = 0;
       for (comp = s1; *comp; comp++)
