@@ -23,9 +23,7 @@
 #define GNUPG_COMMON_HTTP_H 
 
 #include <gpg-error.h>
-#ifdef HTTP_USE_ESTREAM
 #include "estream.h"
-#endif
 
 struct uri_tuple_s {
   struct uri_tuple_s *next;
@@ -64,10 +62,11 @@ http_req_t;
 enum
   { 
     HTTP_FLAG_TRY_PROXY = 1,
-    HTTP_FLAG_NO_SHUTDOWN = 2,
+    HTTP_FLAG_SHUTDOWN = 2,
     HTTP_FLAG_TRY_SRV = 4,
     HTTP_FLAG_LOG_RESP = 8,
-    HTTP_FLAG_NEED_HEADER = 16
+    HTTP_FLAG_NEED_HEADER = 16,
+    HTTP_FLAG_IGNORE_CL = 32
   };
 
 struct http_context_s;
@@ -99,13 +98,8 @@ gpg_error_t http_open_document (http_t *r_hd,
                                 const char *proxy,
                                 void *tls_context);
 
-#ifdef HTTP_USE_ESTREAM
 estream_t http_get_read_ptr (http_t hd);
 estream_t http_get_write_ptr (http_t hd);
-#else /*!HTTP_USE_ESTREAM*/
-FILE *http_get_read_ptr (http_t hd);
-FILE *http_get_write_ptr (http_t hd);
-#endif /*!HTTP_USE_ESTREAM*/
 unsigned int http_get_status_code (http_t hd);
 const char *http_get_header (http_t hd, const char *name);
 
