@@ -92,9 +92,13 @@ software is provided ``as is'' without express or implied warranty.  */
 #define LDAP_REALLOC		realloc
 #define ldap_utf8_strchr	strchr
 #define ldap_utf8_strtok(n,d,s) strtok (n,d)
-#define Debug(a,b,c,d,e)	
+#define Debug(a,b,c,d,e)
 void ldap_pvt_hex_unescape( char *s );
 
+#ifndef LDAP_SCOPE_DEFAULT
+# define LDAP_SCOPE_DEFAULT -1
+#endif
+
 
 
 /* $OpenLDAP: pkg/ldap/libraries/libldap/charray.c,v 1.9.2.2 2003/03/03 17:10:04 kurt Exp $ */
@@ -336,7 +340,7 @@ char * ldap_charray2str( char **a, const char *sep )
 	s = LDAP_MALLOC ( len + 1 );
 
 	if ( s == NULL ) {
-		return NULL;	
+		return NULL;
 	}
 
 	p = s;
@@ -636,7 +640,7 @@ ldap_url_parse_ext( LDAP_CONST char *url_in, LDAPURLDesc **ludpp )
 	 * anything real.
 	 */
 	if( (p == NULL) && (q != NULL) && ((q = strchr( q, '?')) != NULL)) {
-		q++;		
+		q++;
 		/* ? immediately followed by question */
 		if( *q == '?') {
 			q++;
@@ -866,7 +870,7 @@ ldap_free_urldesc( LDAPURLDesc *ludp )
 	if ( ludp == NULL ) {
 		return;
 	}
-	
+
 	if ( ludp->lud_scheme != NULL ) {
 		LDAP_FREE( ludp->lud_scheme );
 	}
