@@ -107,6 +107,10 @@ static struct wrapper_context_s *wrapper_list;
 /* We need to know whether we are shutting down the process */
 static int shutting_down;
 
+/* Close the pth file descriptor FD and set it to -1.  */
+#define SAFE_PTH_CLOSE(fd) \
+  do { int _fd = fd; if (_fd != -1) { pth_close (_fd); fd = -1;} } while (0)
+
 
 /* Prototypes.  */
 static gpg_error_t read_buffer (ksba_reader_t reader,
@@ -114,7 +118,6 @@ static gpg_error_t read_buffer (ksba_reader_t reader,
 
 
 
-
 
 /* Add HOST and PORT to our list of LDAP servers.  Fixme: We should
    better use an extra list of servers. */
@@ -173,10 +176,8 @@ destroy_wrapper (struct wrapper_context_s *ctx)
       dirmngr_release_process (ctx->pid);
     }
   ksba_reader_release (ctx->reader);
-  if (ctx->fd != -1)
-    pth_close (ctx->fd);
-  if (ctx->log_fd != -1)
-    pth_close (ctx->log_fd);
+  SAFE_PTH_CLOSE (ctx->fd);
+  SAFE_PTH_CLOSE (ctx->log_fd);
   if (ctx->log_ev)
     pth_event_free (ctx->log_ev, PTH_FREE_THIS);
   xfree (ctx->line);
@@ -243,6 +244,38 @@ print_log_line (struct wrapper_context_s *ctx, char *line)
 }
 
 
+/* Read data from the log stream.  Returns true if the log stream
+   indicated EOF or error.  */
+static int
+read_log_data (struct wrapper_context_s *ctx)
+{
+  int n;
+  char line[256];
+
+  /* We must use the pth_read function for pipes, always.  */
+  do 
+    n = pth_read (ctx->log_fd, line, sizeof line - 1);
+  while (n < 0 && errno == EINTR);
+
+  if (n <= 0) /* EOF or error. */
+    {
+      if (n < 0)
+        log_error (_("error reading log from ldap wrapper %d: %s\n"),
+                   ctx->pid, strerror (errno));
+      print_log_line (ctx, NULL);
+      SAFE_PTH_CLOSE (ctx->log_fd);
+      pth_event_free (ctx->log_ev, PTH_FREE_THIS);
+      ctx->log_ev = NULL;
+      return 1;
+    }
+
+  line[n] = 0;
+  print_log_line (ctx, line);
+  if (ctx->stamp != (time_t)(-1))
+    ctx->stamp = time (NULL);
+  return 0;
+}
+
 
 /* This function is run by a separate thread to maintain the list of
    wrappers and to log error messages from these wrappers. */
@@ -250,10 +283,8 @@ void *
 ldap_wrapper_thread (void *dummy)
 {
   int nfds;
-  int n;
   struct wrapper_context_s *ctx;
   struct wrapper_context_s *ctx_prev;
-  char line[256];
   time_t current_time;
 
   (void)dummy;
@@ -262,6 +293,7 @@ ldap_wrapper_thread (void *dummy)
   for (;;)
     {
       pth_event_t timeout_ev;
+      int any_action = 0;
 
       timeout_ev = pth_event (PTH_EVENT_TIME, pth_timeout (1, 0));
       if (! timeout_ev)
@@ -309,31 +341,8 @@ ldap_wrapper_thread (void *dummy)
           if (nfds && ctx->log_fd != -1
 	      && pth_event_status (ctx->log_ev) == PTH_STATUS_OCCURRED)
             {
-              /* We must use the pth_read function for pipes, always.  */
-              do 
-                n = pth_read (ctx->log_fd, line, sizeof line - 1);
-              while (n < 0 && errno == EINTR);
-              if (n < 0)
-                {
-                  print_log_line (ctx, NULL);
-                  log_error (_("error reading log from ldap wrapper %d: %s\n"),
-                             ctx->pid, strerror (errno));
-                }
-              else if (!n) /* EOF */
-                {
-                  print_log_line (ctx, NULL);
-                  pth_close (ctx->log_fd);
-                  ctx->log_fd = -1;
-                  pth_event_free (ctx->log_ev, PTH_FREE_THIS);
-                  ctx->log_ev = NULL;
-                }
-              else 
-                {
-                  line[n] = 0;
-                  print_log_line (ctx, line);
-                  if (ctx->stamp != (time_t)(-1))
-                    ctx->stamp = time (NULL);
-                }
+              if (read_log_data (ctx))
+                any_action = 1;
             }
 
           /* Check whether the process is still running.  */
@@ -351,11 +360,13 @@ ldap_wrapper_thread (void *dummy)
                   ctx->ready = 1;
 		  dirmngr_release_process (ctx->pid);
                   ctx->pid = (pid_t)(-1);
+                  any_action = 1;
                 }
               else if (gpg_err_code (err) != GPG_ERR_TIMEOUT)
                 {
                   log_error (_("waiting for ldap wrapper %d failed: %s\n"),
                              (int)ctx->pid, gpg_strerror (err));
+                  any_action = 1;
                 }
             }
 
@@ -367,15 +378,36 @@ ldap_wrapper_thread (void *dummy)
                 ctx->stamp = (time_t)(-1);
               log_info (_("ldap wrapper %d stalled - killing\n"),
                         (int)ctx->pid);
+              /* We need to close the log fd because the cleanup loop
+                 waits for it.  */
+              SAFE_PTH_CLOSE (ctx->log_fd);
+              any_action = 1;
             }
         }
 
+      /* If something has been printed to the log file or we got an
+         EOF from a wrapper, we now print the list of active
+         wrappers.  */
+      if (any_action && DBG_LOOKUP) 
+        {
+          log_info ("ldap worker stati:\n");
+          for (ctx = wrapper_list; ctx; ctx = ctx->next)
+            log_info ("  c=%p pid=%d/%d rdr=%p ctrl=%p/%d la=%lu rdy=%d\n",
+                      ctx, 
+                      (int)ctx->pid, (int)ctx->printable_pid,
+                      ctx->reader,
+                      ctx->ctrl, ctx->ctrl? ctx->ctrl->refcount:0, 
+                      (unsigned long)ctx->stamp, ctx->ready);
+        }
+
+
       /* Use a separate loop to check whether ready marked wrappers
          may be removed.  We may only do so if the ksba reader object
          is not anymore in use or we are in shutdown state.  */
      again:
       for (ctx_prev=NULL, ctx=wrapper_list; ctx; ctx_prev=ctx, ctx=ctx->next)
-        if (ctx->ready && (!ctx->reader || shutting_down))
+        if (ctx->ready 
+            && ((ctx->log_fd == -1 && !ctx->reader) || shutting_down))
           {
             if (ctx_prev)
               ctx_prev->next = ctx->next;
@@ -409,7 +441,6 @@ void
 ldap_wrapper_release_context (ksba_reader_t reader)
 {
   struct wrapper_context_s *ctx;
-  int fd;
 
   if (!reader )
     return;
@@ -417,13 +448,15 @@ ldap_wrapper_release_context (ksba_reader_t reader)
   for (ctx=wrapper_list; ctx; ctx=ctx->next)
     if (ctx->reader == reader)
       {
+        if (DBG_LOOKUP)
+          log_info ("releasing ldap worker c=%p pid=%d/%d rdr=%p ctrl=%p/%d\n",
+                    ctx, 
+                    (int)ctx->pid, (int)ctx->printable_pid,
+                    ctx->reader,
+                    ctx->ctrl, ctx->ctrl? ctx->ctrl->refcount:0);
+
         ctx->reader = NULL;
-        if (ctx->fd != -1)
-          {
-            fd = ctx->fd;
-            ctx->fd = -1;
-            pth_close (fd);
-          }
+        SAFE_PTH_CLOSE (ctx->fd);
         if (ctx->ctrl)
           {
             ctx->ctrl->refcount--;
@@ -498,8 +531,7 @@ reader_callback (void *cb_value, char *buffer, size_t count,  size_t *nread)
           if (err)
             {
               ctx->fd_error = err;
-              pth_close (ctx->fd);
-              ctx->fd = -1;
+              SAFE_PTH_CLOSE (ctx->fd);
               if (evt)
                 pth_event_free (evt, PTH_FREE_THIS);
               return -1;
@@ -509,8 +541,7 @@ reader_callback (void *cb_value, char *buffer, size_t count,  size_t *nread)
       else if (n < 0)
         {
           ctx->fd_error = gpg_error_from_errno (errno);
-          pth_close (ctx->fd);
-          ctx->fd = -1;
+          SAFE_PTH_CLOSE (ctx->fd);
           if (evt)
             pth_event_free (evt, PTH_FREE_THIS);
           return -1;
@@ -589,8 +620,8 @@ ldap_wrapper (ctrl_t ctrl, ksba_reader_t *reader, const char *argv[])
      everything gets intermixed.  Clearly we don't want this.  So the
      only viable solutions are either to have another thread
      responsible for logging the messages or to add an option to the
-     wrapper modules to the logging on its own.  Given that we anyway
-     need a way to rip the child process and this best done using a
+     wrapper module to do the logging on its own.  Given that we anyway
+     need a way to rip the child process and this is best done using a
      general ripping thread, that thread can do the logging too. */
 
   *reader = NULL;
@@ -1420,7 +1451,11 @@ end_cert_fetch_ldap (cert_fetch_context_t context)
 {
   if (context)
     {
+      ksba_reader_t reader = context->reader;
+
       xfree (context->tmpbuf);
       xfree (context);
+      ldap_wrapper_release_context (reader);
+      ksba_reader_release (reader);
     }
 }
