@@ -1,6 +1,6 @@
 /* dirmngr.c - LDAP access
  *	Copyright (C) 2002 Klarälvdalens Datakonsult AB
- *      Copyright (C) 2003, 2004, 2005, 2007, 2008 g10 Code GmbH
+ *      Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009 g10 Code GmbH
  *
  * This file is part of DirMngr.
  *
@@ -416,7 +416,7 @@ inquire_cert_and_load_crl (assuan_context_t ctx)
 
 
 /* Handle OPTION commands. */
-static int
+static gpg_error_t
 option_handler (assuan_context_t ctx, const char *key, const char *value)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -437,8 +437,12 @@ option_handler (assuan_context_t ctx, const char *key, const char *value)
   return 0;
 }
 
-
-static int
+static const char hlp_ldapserver[] = 
+  "LDAPSERVER <data>\n"
+  "\n"
+  "Add a new LDAP server to the list of configured LDAP servers.\n"
+  "DATA is in the same format as expected in the configure file.";
+static gpg_error_t
 cmd_ldapserver (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -462,31 +466,30 @@ cmd_ldapserver (assuan_context_t ctx, char *line)
 }
 
 
-/* ISVALID [--only-ocsp] [--force-default-responder]
-            <certificate_id>|<certificate_fpr>
-  
-   This command checks whether the certificate identified by the
-   certificate_id is valid.  This is done by consulting CRLs or
-   whatever has been configured.  Note, that the returned error codes
-   are from gpg-error.h.  The command may callback using the inquire
-   function.  See the manual for details.
- 
-   The CERTIFICATE_ID is a hex encoded string consisting of two parts,
-   delimited by a single dot.  The first part is the SHA-1 hash of the
-   issuer name and the second part the serial number.
-
-   Alternatively the certificate's fingerprint may be given in which
-   case an OCSP request is done before consulting the CRL.
-
-   If the option --only-ocsp is given, no fallback to a CRL check will
-   be used.
-
-   If the option --force-default-responder is given, only the default
-   OCSP responder will be used and any other methods of obtaining an
-   OCSP responder URL won't be used.
- */
-
-static int
+static const char hlp_isvalid[] = 
+  "ISVALID [--only-ocsp] [--force-default-responder]"
+  " <certificate_id>|<certificate_fpr>\n"
+  "\n"
+  "This command checks whether the certificate identified by the\n"
+  "certificate_id is valid.  This is done by consulting CRLs or\n"
+  "whatever has been configured.  Note, that the returned error codes\n"
+  "are from gpg-error.h.  The command may callback using the inquire\n"
+  "function.  See the manual for details.\n"
+  "\n"
+  "The CERTIFICATE_ID is a hex encoded string consisting of two parts,\n"
+  "delimited by a single dot.  The first part is the SHA-1 hash of the\n"
+  "issuer name and the second part the serial number.\n"
+  "\n"
+  "Alternatively the certificate's fingerprint may be given in which\n"
+  "case an OCSP request is done before consulting the CRL.\n"
+  "\n"
+  "If the option --only-ocsp is given, no fallback to a CRL check will\n"
+  "be used.\n"
+  "\n"
+  "If the option --force-default-responder is given, only the default\n"
+  "OCSP responder will be used and any other methods of obtaining an\n"
+  "OCSP responder URL won't be used.";
+static gpg_error_t
 cmd_isvalid (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -615,27 +618,27 @@ get_fingerprint_from_line (const char *line, unsigned char *fpr)
 
 
 
-/* CHECKCRL [<fingerprint>]
-
-   Check whether the certificate with FINGERPRINT (SHA-1 hash of the
-   entire X.509 certificate blob) is valid or not by consulting the
-   CRL responsible for this certificate.  If the fingerprint has not
-   been given or the certificate is not known, the function 
-   inquires the certificate using an
-
-      INQUIRE TARGETCERT
-
-   and the caller is expected to return the certificate for the
-   request (which should match FINGERPRINT) as a binary blob.
-   Processing then takes place without further interaction; in
-   particular dirmngr tries to locate other required certificate by
-   its own mechanism which includes a local certificate store as well
-   as a list of trusted root certificates.
-
-   The return value is the usual gpg-error code or 0 for ducesss;
-   i.e. the certificate validity has been confirmed by a valid CRL.
-*/
-static int
+static const char hlp_checkcrl[] = 
+  "CHECKCRL [<fingerprint>]\n"
+  "\n"
+  "Check whether the certificate with FINGERPRINT (SHA-1 hash of the\n"
+  "entire X.509 certificate blob) is valid or not by consulting the\n"
+  "CRL responsible for this certificate.  If the fingerprint has not\n"
+  "been given or the certificate is not known, the function \n"
+  "inquires the certificate using an\n"
+  "\n"
+  "  INQUIRE TARGETCERT\n"
+  "\n"
+  "and the caller is expected to return the certificate for the\n"
+  "request (which should match FINGERPRINT) as a binary blob.\n"
+  "Processing then takes place without further interaction; in\n"
+  "particular dirmngr tries to locate other required certificate by\n"
+  "its own mechanism which includes a local certificate store as well\n"
+  "as a list of trusted root certificates.\n"
+  "\n"
+  "The return value is the usual gpg-error code or 0 for ducesss;\n"
+  "i.e. the certificate validity has been confirmed by a valid CRL.";
+static gpg_error_t
 cmd_checkcrl (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -692,34 +695,34 @@ cmd_checkcrl (assuan_context_t ctx, char *line)
 }
 
 
-/* CHECKOCSP [--force-default-responder] [<fingerprint>]
-
-   Check whether the certificate with FINGERPRINT (SHA-1 hash of the
-   entire X.509 certificate blob) is valid or not by asking an OCSP
-   responder responsible for this certificate.  The optional
-   fingerprint may be used for a quick check in case an OCSP check has
-   been done for this certificate recently (we always cache OCSP
-   responses for a couple of minutes). If the fingerprint has not been
-   given or there is no cached result, the function inquires the
-   certificate using an
-
-      INQUIRE TARGETCERT
-
-   and the caller is expected to return the certificate for the
-   request (which should match FINGERPRINT) as a binary blob.
-   Processing then takes place without further interaction; in
-   particular dirmngr tries to locate other required certificates by
-   its own mechanism which includes a local certificate store as well
-   as a list of trusted root certifciates.
-
-   If the option --force-default-responder is given, only the default
-   OCSP responder will be used and any other methods of obtaining an
-   OCSP responder URL won't be used.
-
-   The return value is the usual gpg-error code or 0 for ducesss;
-   i.e. the certificate validity has been confirmed by a valid CRL.
-*/
-static int
+static const char hlp_checkocsp[] = 
+  "CHECKOCSP [--force-default-responder] [<fingerprint>]\n"
+  "\n"
+  "Check whether the certificate with FINGERPRINT (SHA-1 hash of the\n"
+  "entire X.509 certificate blob) is valid or not by asking an OCSP\n"
+  "responder responsible for this certificate.  The optional\n"
+  "fingerprint may be used for a quick check in case an OCSP check has\n"
+  "been done for this certificate recently (we always cache OCSP\n"
+  "responses for a couple of minutes). If the fingerprint has not been\n"
+  "given or there is no cached result, the function inquires the\n"
+  "certificate using an\n"
+  "\n"
+  "   INQUIRE TARGETCERT\n"
+  "\n"
+  "and the caller is expected to return the certificate for the\n"
+  "request (which should match FINGERPRINT) as a binary blob.\n"
+  "Processing then takes place without further interaction; in\n"
+  "particular dirmngr tries to locate other required certificates by\n"
+  "its own mechanism which includes a local certificate store as well\n"
+  "as a list of trusted root certifciates.\n"
+  "\n"
+  "If the option --force-default-responder is given, only the default\n"
+  "OCSP responder will be used and any other methods of obtaining an\n"
+  "OCSP responder URL won't be used.\n"
+  "\n"
+  "The return value is the usual gpg-error code or 0 for ducesss;\n"
+  "i.e. the certificate validity has been confirmed by a valid CRL.";
+static gpg_error_t
 cmd_checkocsp (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -1024,28 +1027,26 @@ lookup_cert_by_pattern (assuan_context_t ctx, char *line,
 }
 
 
-/* LOOKUP [--url] [--single] [--cache-only] <pattern>
-
-   Lookup certificates matching PATTERN. With --url the pattern is
-   expected to be one URL.
-
-   If --url is not given:  To allow for multiple
-   patterns (which are ORed) quoting is required: Spaces are to be
-   translated into "+" or into "%20"; obviously this requires that the
-   usual escape quoting rules are applied.
-
-   If --url is given no special escaping is required because URLs are
-   already escaped this way.
-
-   if --single is given the first and only the first match will be
-   returned.  If --cache-only is _not_ given, no local query will be
-   done.
-
-   if --cache-only is given no external lookup is done so that only
-   certificates from the cache may get returned.
-*/
-
-static int
+static const char hlp_lookup[] = 
+  "LOOKUP [--url] [--single] [--cache-only] <pattern>\n"
+  "\n"
+  "Lookup certificates matching PATTERN. With --url the pattern is\n"
+  "expected to be one URL.\n"
+  "\n"
+  "If --url is not given:  To allow for multiple patterns (which are ORed)\n"
+  "quoting is required: Spaces are translated to \"+\" or \"%20\";\n"
+  "obviously this requires that the usual escape quoting rules are applied.\n"
+  "\n"
+  "If --url is given no special escaping is required because URLs are\n"
+  "already escaped this way.\n"
+  "\n"
+  "If --single is given the first and only the first match will be\n"
+  "returned.  If --cache-only is _not_ given, no local query will be\n"
+  "done.\n"
+  "\n"
+  "If --cache-only is given no external lookup is done so that only\n"
+  "certificates from the cache may get returned.";
+static gpg_error_t
 cmd_lookup (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1072,42 +1073,73 @@ cmd_lookup (assuan_context_t ctx, char *line)
 }
 
 
-/* LOADCRL <filename>
-
-   Load the CRL in the file with name FILENAME into our cache.  Note
-   that FILENAME should be given with an absolute path because
-   Dirmngrs cwd is not known.  This command is usually used by gpgsm
-   using the invocation "gpgsm --call-dirmngr loadcrl <filename>".  A
-   direct invocation of Dirmngr is not useful because gpgsm might need
-   to callback gpgsm to ask for the CA's certificate.
-*/
-
-static int
+static const char hlp_loadcrl[] =
+  "LOADCRL [--url] <filename|url>\n"
+  "\n"
+  "Load the CRL in the file with name FILENAME into our cache.  Note\n"
+  "that FILENAME should be given with an absolute path because\n"
+  "Dirmngrs cwd is not known.  With --url the CRL is directly loaded\n"
+  "from the given URL.\n"
+  "\n"
+  "This command is usually used by gpgsm using the invocation \"gpgsm\n"
+  "--call-dirmngr loadcrl <filename>\".  A direct invocation of Dirmngr\n"
+  "is not useful because gpgsm might need to callback gpgsm to ask for\n"
+  "the CA's certificate.";
+static gpg_error_t
 cmd_loadcrl (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
-  gpg_error_t err;
-  char *buf;
+  gpg_error_t err = 0;
+  int use_url = has_leading_option (line, "--url");
+
+  line = skip_options (line);
+
+  if (use_url)
+    {
+      ksba_reader_t reader;
+
+      err = crl_fetch (ctrl, line, &reader);
+      if (err)
+        log_error (_("fetching CRL from `%s' failed: %s\n"),
+                   line, gpg_strerror (err));
+      else
+        {
+          err = crl_cache_insert (ctrl, line, reader); 
+          if (err)
+            log_error (_("processing CRL from `%s' failed: %s\n"),
+                       line, gpg_strerror (err));
+          crl_close_reader (reader);
+        }
+    }
+  else
+    {
+      char *buf;
+
+      buf = xtrymalloc (strlen (line)+1);
+      if (!buf)
+        err = gpg_error_from_syserror ();
+      else
+        {
+          strcpy_escaped_plus (buf, line);
+          err = crl_cache_load (ctrl, buf);
+          xfree (buf);
+        }
+    }
 
-  buf = xmalloc (strlen (line)+1);
-  strcpy_escaped_plus (buf, line);
-  err = crl_cache_load (ctrl, buf);
-  xfree (buf);
   if (err)
     log_error (_("command %s failed: %s\n"), "LOADCRL", gpg_strerror (err));
   return err;
 }
 
 
-/* LISTCRLS 
-
-   List the content of all CRLs in a readable format.  This command is
-   usually used by gpgsm using the invocation "gpgsm --call-dirmngr
-   listcrls".  It may also be used directly using "dirmngr
-   --list-crls".
-*/
-
-static int
+static const char hlp_listcrls[] =
+  "LISTCRLS\n"
+  "\n"
+  "List the content of all CRLs in a readable format.  This command is\n"
+  "usually used by gpgsm using the invocation \"gpgsm --call-dirmngr\n"
+  "listcrls\".  It may also be used directly using \"dirmngr\n"
+  "--list-crls\".";
+static gpg_error_t
 cmd_listcrls (assuan_context_t ctx, char *line)
 {
   gpg_error_t err;
@@ -1125,20 +1157,20 @@ cmd_listcrls (assuan_context_t ctx, char *line)
 }
 
 
-/* CACHECERT 
-
-   Put a certificate into the internal cache.  This command might be
-   useful if a client knows in advance certificates required for a
-   test and wnats to make sure they get added to the internal cache.
-   It is also helpful for debugging.  To get the actual certificate,
-   this command immediately inquires it using
-
-      INQUIRE TARGETCERT
-
-   and the caller is expected to return the certificate for the
-   request as a binary blob.
-*/
-static int
+static const char hlp_cachecert[] = 
+  "CACHECERT\n"
+  "\n"
+  "Put a certificate into the internal cache.  This command might be\n"
+  "useful if a client knows in advance certificates required for a\n"
+  "test and wnats to make sure they get added to the internal cache.\n"
+  "It is also helpful for debugging.  To get the actual certificate,\n"
+  "this command immediately inquires it using\n"
+  "\n"
+  "  INQUIRE TARGETCERT\n"
+  "\n"
+  "and the caller is expected to return the certificate for the\n"
+  "request as a binary blob.";
+static gpg_error_t
 cmd_cachecert (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -1179,19 +1211,19 @@ cmd_cachecert (assuan_context_t ctx, char *line)
 }
 
 
-/* VALIDATE 
-
-   Validate a certificate using the certificate validation function
-   used internally by dirmngr.  This command is only useful for
-   debugging.  To get the actual certificate, this command immediately
-   inquires it using
-
-      INQUIRE TARGETCERT
-
-   and the caller is expected to return the certificate for the
-   request as a binary blob.
-*/
-static int
+static const char hlp_validate[] =
+  "VALIDATE\n"
+  "\n"
+  "Validate a certificate using the certificate validation function\n"
+  "used internally by dirmngr.  This command is only useful for\n"
+  "debugging.  To get the actual certificate, this command immediately\n"
+  "inquires it using\n"
+  "\n"
+  "  INQUIRE TARGETCERT\n"
+  "\n"
+  "and the caller is expected to return the certificate for the\n"
+  "request as a binary blob.";
+static gpg_error_t
 cmd_validate (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
@@ -1255,17 +1287,18 @@ register_commands (assuan_context_t ctx)
 {
   static struct {
     const char *name;
-    int (*handler)(assuan_context_t, char *line);
+    assuan_handler_t handler;
+    const char * const help;
   } table[] = {
-    { "LDAPSERVER", cmd_ldapserver },
-    { "ISVALID",    cmd_isvalid },
-    { "CHECKCRL",   cmd_checkcrl },
-    { "CHECKOCSP",  cmd_checkocsp },
-    { "LOOKUP",     cmd_lookup },
-    { "LOADCRL",    cmd_loadcrl },
-    { "LISTCRLS",   cmd_listcrls },
-    { "CACHECERT",  cmd_cachecert },
-    { "VALIDATE",   cmd_validate },
+    { "LDAPSERVER", cmd_ldapserver, hlp_ldapserver },
+    { "ISVALID",    cmd_isvalid,    hlp_isvalid },
+    { "CHECKCRL",   cmd_checkcrl,   hlp_checkcrl },
+    { "CHECKOCSP",  cmd_checkocsp,  hlp_checkocsp },
+    { "LOOKUP",     cmd_lookup,     hlp_lookup },
+    { "LOADCRL",    cmd_loadcrl,    hlp_loadcrl },
+    { "LISTCRLS",   cmd_listcrls,   hlp_listcrls },
+    { "CACHECERT",  cmd_cachecert,  hlp_cachecert },
+    { "VALIDATE",   cmd_validate,   hlp_validate },
     { "INPUT",      NULL },
     { "OUTPUT",     NULL },
     { NULL, NULL }
@@ -1274,7 +1307,8 @@ register_commands (assuan_context_t ctx)
 
   for (i=j=0; table[i].name; i++)
     {
-      rc = assuan_register_command (ctx, table[i].name, table[i].handler);
+      rc = assuan_register_command (ctx, table[i].name, table[i].handler,
+                                    table[i].help);
       if (rc)
         return rc;
     }
@@ -1282,13 +1316,14 @@ register_commands (assuan_context_t ctx)
 }
 
 
-static void
-reset_notify (assuan_context_t ctx)
+static gpg_error_t
+reset_notify (assuan_context_t ctx, char *line)
 {
   ctrl_t ctrl = assuan_get_pointer (ctx);
 
   ldapserver_list_free (ctrl->server_local->ldapservers);
   ctrl->server_local->ldapservers = NULL;
+  return 0;
 }
 
 
@@ -1316,21 +1351,30 @@ start_command_handler (assuan_fd_t fd)
     
   dirmngr_init_default_ctrl (ctrl);
 
+  rc = assuan_new (&ctx);
+  if (rc)
+    {
+      log_error (_("failed to allocate assuan context: %s\n"),
+		 gpg_strerror (rc));
+      dirmngr_exit (2);
+    }
+
   if (fd == ASSUAN_INVALID_FD)
     {
-      int filedes[2];
-
-      filedes[0] = 0;
-      filedes[1] = 1;
-      rc = assuan_init_pipe_server (&ctx, filedes);
+      assuan_fd_t filedes[2];
+      
+      filedes[0] = assuan_fdopen (0);
+      filedes[1] = assuan_fdopen (1);
+      rc = assuan_init_pipe_server (ctx, filedes);
     }
   else
     {
-      rc = assuan_init_socket_server_ext (&ctx, fd, 2);
+      rc = assuan_init_socket_server (ctx, fd, ASSUAN_SOCKET_SERVER_ACCEPTED);
     }
 
   if (rc)
     {
+      assuan_release (ctx);
       log_error (_("failed to initialize the server: %s\n"),
                  gpg_strerror(rc));
       dirmngr_exit (2);
@@ -1389,13 +1433,12 @@ start_command_handler (assuan_fd_t fd)
 #ifndef HAVE_W32_SYSTEM
       if (opt.verbose)
         {
-          pid_t apid;
-          uid_t auid;
-          gid_t agid;
+	  assuan_peercred_t peercred;
 
-          if (!assuan_get_peercred (ctx, &apid, &auid, &agid))
+          if (!assuan_get_peercred (ctx, &peercred))
             log_info ("connection from process %ld (%ld:%ld)\n",
-                      (long)apid, (long)auid, (long)agid);
+                      (long)peercred->pid, (long)peercred->uid,
+		      (long)peercred->gid);
         }
 #endif
 
@@ -1413,7 +1456,7 @@ start_command_handler (assuan_fd_t fd)
   ctrl->server_local->ldapservers = NULL;
 
   ctrl->server_local->assuan_ctx = NULL;
-  assuan_deinit_server (ctx);
+  assuan_release (ctx);
 
   if (ctrl->refcount)
     log_error ("oops: connection control structure still referenced (%d)\n",
