@@ -78,6 +78,8 @@ unknown_criticals (ksba_cert_t cert)
   };
   int i, idx, crit;
   const char *oid;
+  int unsupported;
+  strlist_t sl;
   gpg_error_t err, rc;
 
   rc = 0;
@@ -88,7 +90,21 @@ unknown_criticals (ksba_cert_t cert)
         continue;
       for (i=0; known[i] && strcmp (known[i],oid); i++)
         ;
-      if (!known[i])
+      unsupported = !known[i];
+
+      /* If this critical extension is not supported, check the list
+         of to be ignored extensions to see whether we claim that it
+         is supported.  */
+      if (unsupported && opt.ignored_cert_extensions)
+        {
+          for (sl=opt.ignored_cert_extensions;
+               sl && strcmp (sl->d, oid); sl = sl->next)
+            ;
+          if (sl)
+            unsupported = 0;
+        }
+
+      if (unsupported)
         {
           log_error (_("critical certificate extension %s is not supported"),
                      oid);
